from xml.dom.minidom import *
from xml.dom.minidom import Document
from random import Random
from string import strip,atoi

TYPE_LIST = ["REQ", "REP"]

class XmlMessage:
	"""
	XmlMessage
	
	This module creates a xml message or accesses its content.
	
	Members:
	
	XmlMessage:XmlMessage()
	XmlMessage:getBody()
	XmlMessage:getSender()
	XmlMessage:getSeq()
	XmlMessage:getType()

	XmlMessage:setData_req()
	XmlMessage:setData_rep()
	"""

	def __init__(self, string=None):
		"""
		XmlMessage(string: XmlString)
		
		Constructor, creates a xmlMessage.
		
		XmlMessage has two uses: interpret an existing xml string or create a new
		xml string.
		
		In the first case, the following command should be used, in order to create a XmlMessage:
		msg = XmlMessage(xml_string)
		
		Otherwise, in case you want to create a new xml, the following might be used:
		msg = XmlMessage()
		
		In this case, a simple structure will be created:
			<msg>
				<type>  </type>
			</msg>
		"""
		if string:
			# in case a xml string is given:
			self.document = parseString(string)
			# I need to ransform this xml.dom.minidom.Document into a XmlMessage
		else:
			# otherwise
			self.document = Document()
			msg = self.document.createElement("msg")
			self.document.appendChild(msg)
			
			typeField = self.document.createElement("type")
			msg.appendChild(typeField)
			
			self.msg       = msg
			self.typeField = typeField


	def setData_req(self, site_id=0, site_seq=0, body = 0):
		"""
		XmlMessage.setData_req(site_id: int, site_seq: int)
		
		Creates fields related to request message, and put the related data inside them.
		
		Params:
			site_id  - sender site identification, from 1 to N (number of sites)
			site_seq - site sequency number, in the interval [1..1024]
			body     - body is a random number from [1..100000]
		"""
		
		msg       = self.msg
		typeField = self.typeField
		
		type_text = self.document.createTextNode("REQ")
		typeField.appendChild(type_text)
		
		siteIdField = self.document.createElement("site_id")
		msg.appendChild(siteIdField)
		
		if site_id:
			site_id_number = self.document.createTextNode(str(site_id))
			siteIdField.appendChild(site_id_number)
		
		siteSeqField = self.document.createElement("site_seq")
		msg.appendChild(siteSeqField)
		
		if (0 < site_seq < 1024):
			site_seq = int(site_seq)
			site_seq_number = self.document.createTextNode(str(site_seq))
			siteSeqField.appendChild(site_seq_number)
		elif (site_seq >= 1024) or (site_seq < 0):
			debug("ERROR - Site sequency value should be in range [1; 1024]")
			return 0
		
		bodyField = self.document.createElement("body")
		msg.appendChild(bodyField)
		
		if (0 < body <= 100000):
			body = int(body)
			body_text = self.document.createTextNode(str(body))
			bodyField.appendChild(body_text)
		elif (body > 100000) or (body < 0):
			debug("ERROR - Body value should be in range [1; 100000]")
			return 0
		
		self.msg        = msg


	def setData_rep(self, site_id=0, site_seq=0, body = 0):
		"""
		XmlMessage.setData_rep(site_id: int, site_seq: int)
		
		Creates fields related to response message, and put the related data inside them.
		
		Params:
			site_id  - sender site identification, from 1 to N (number of sites)
			site_seq - site sequency number, in the interval [1..1024]
			body     - body is a random number from [1..100000]
		"""
		
		msg       = self.msg
		typeField = self.typeField
		
		type_text = self.document.createTextNode("REP")
		typeField.appendChild(type_text)
		
		siteIdField = self.document.createElement("site_id")
		msg.appendChild(siteIdField)
		
		if site_id:
			site_id_number = self.document.createTextNode(str(site_id))
			siteIdField.appendChild(site_id_number)
		
		siteSeqField = self.document.createElement("site_seq")
		msg.appendChild(siteSeqField)
		
		if (0 < site_seq < 1024):
			site_seq = int(site_seq)
			site_seq_number = self.document.createTextNode(str(site_seq))
			siteSeqField.appendChild(site_seq_number)
		elif (site_seq >= 1024) or (site_seq < 0):
			debug("ERROR - Site sequency value should be in range [1; 1024]")
			return 0
		
		bodyField = self.document.createElement("body")
		msg.appendChild(bodyField)
		
		if (0 < body <= 100000):
			body = int(body)
			body_text = self.document.createTextNode(str(body))
			bodyField.appendChild(body_text)
		elif (body > 100000) or (body < 0):
			debug("ERROR - Body value should be in range [1; 100000]")
			return 0
		
		self.msg        = msg

	def getData(self, field):
		"""
		XmlMessage.getData(field:MSG_FIELD)
		
		Retrieves the information in the specified field.
		
		Params:
			field     - the field where the data you're interested is
		
		Returns:
			element in field - which will be a string, in general
		"""
		
		nos = self.document.documentElement
		
		filhos1 = [no for no in nos.childNodes if no.nodeType == self.document.ELEMENT_NODE]

		for filho in filhos1:
			if filho.nodeName == field:
				return self._getValue(filho)
			
	def _getValue(self, node):
		text = ''
		for child in node.childNodes:
			if child.nodeType is child.TEXT_NODE:
				text += child.data
		
		return strip(text)

	def getText(self):
		"""
		XmlMessage.getText()
		
		Retrieves a xml encoded string with the instance data.
		"""
		return self.document.toxml() # non idented block
		#return self.document.toprettyxml() # idented block

	def getType(self):
		"""
		XmlMessage.getType()
		
		Returns message type.
		"""
		return strip(self.getData("type"))

	def getSender(self):
		"""
		XmlMessage.getSender()
		
		Returns sender's site identification.
		"""
		return atoi(self.getData("site_id"))

	def getSeqNumber(self):
		"""
		XmlMessage.getSeqNumber()
		
		Returns message's sequency number.
		"""

		return atoi(self.getData("site_seq"))


	def getBody(self):
		"""
		XmlMessage.getBody()
		
		Returns the content (body) of the message.
		"""
		
		return self.getData("body")
